// NEW KEYNESIAN MODEL, with the (stationary) bridge methodology
// BETA version modified by Luis Rojas - Dic 2013 
// The non-model based component is  calibrated

// Model
// IS:  x(t) = E_t{x(t+1)} - 1/phi*( r(t) - E_t{p(t+1)} ) + g(t)
// PC:  p(t) = betta*E_t{p(t+1)} + kappa*x(t) + u(t)
// Taylor-Rule: r(t) = psir*r(t-1) + (1-psir)*(phix*x(t) + psi*p(t)) + er(t)
//
//  E(t)=(g(t), u(t), er(t))
//  E(t) = rho*E(t-1) + A*ee(t);  ee ~ nid(0,sigg^2), 
//
// p = inflation, x = output gap, r = interest rate
//
// kappa = lambda * (phi+nu)
// lambda = (1-theta)*(1-betta*theta)/theta

close all; clc;

% preliminary data management
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load gpr.dat;              % data from Ireland (2002) - downloaded 2013 (LER)
[nobs,nvar] = size(gpr);   % # Observations in time series
y = gpr(:,1);
pp = gpr(:,2);
rr = gpr(:,3);

%******L********L**********L*****************L********L**********L***********
%   This section will be ommited in the following release.
%   The bridge methodology will account for the trends directly.
%******L********L**********L*****************L********L**********L***********
//dates = cal(1948,1,4);
// detrend y, remove linear trend
cr = ones(nobs,1);
tr = (1:nobs)';
for i=1:nobs
tr2(i) = tr(i)^2;
end;
yt = FOLS(y,[cr tr tr2']);
x = y-yt.beta(2)*tr-yt.beta(3)*tr2';       % output gap
pt = FOLS(pp,[cr tr tr2']);
p = pp-pt.beta(1)-pt.beta(2)*tr-pt.beta(3)*tr2';       % eliminate quadratic trend
rt = FOLS(rr,[cr tr tr2']);
r = rr-rt.beta(1)-rt.beta(2)*tr-rt.beta(3)*tr2';       % eliminate quadratic trend


 
save rawdata_US_1948Q1_2004q4_L  r x p;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


var r x p g u eer x_m p_m r_m w_r w_p w_x r_nm x_nm p_nm;    
varexo eg eu er v_11 v_12 v_13 v_21 v_22 v_23 u_r u_x u_p;  
parameters betta theta phi nu psir psix psip rhog rhou sigg sigu sigr 
rho_11 rho_12 rho_13 rho_21 rho_22 rho_23 sig_11 sig_12 sig_13 sig_21 
sig_22 sig_23 sig_ur sig_ux sig_up ;

% these are use as initial values for the parameters 
betta= 0.99;
theta= 0.75;
phi=2.0376;
nu=2.0091;
psix=-0.000;
psir=0.8283;
psip=1.4912;
rhog=0.9848;
rhou=0.9999;

% calibrated parameters (non-model based component)
rho_11=0.5;
rho_12=0.5;
rho_13=0.5;
rho_21=0.5;
rho_22=0.5;
rho_23=0.5;

// Model based component
model(linear); 
          x_m = x_m(1)-(1/phi)*(r_m-p_m(1))+g;
          p_m = betta*p_m(1)+(((1-theta)*(1-betta*theta)/theta)*(phi+nu))*x_m+u;
//        r_m = psir*r_m(-1)+(1-psir)*(psip*p(-1) + psix*x(-1)) + eer;
          r_m = psir*r_m(-1)+(1-psir)*(psip*p_m + psix*x_m) + eer; 
          g = rhog*g(-1)+eg;
          u = rhou*u(-1)+eu;
          eer =0*eer(-1)+er;

// Non-model based component (varobs=r, x, p)
r_nm= rho_11*r_nm(-1)+w_r(-1)+v_11;
w_r=rho_21*w_r(-1)+v_21;
x_nm= rho_12*x_nm(-1)+w_x(-1)+v_12;
w_x=rho_22*w_x(-1)+v_22;
p_nm= rho_13*p_nm(-1)+w_p(-1)+v_13;
w_p=rho_23*w_p(-1)+v_23;

// Measurement equations
r= r_nm + r_m + u_r;
x= x_nm + x_m + u_x;
p= p_nm + p_m + u_p;
end; 

shocks;
var eg; stderr 0.1;
var eu; stderr 0.1;
var er; stderr 0.1;

// shocks of the non modelled component
var v_11; stderr 0.001;
var v_12; stderr 0.001;
var v_13; stderr 0.001;
var v_21; stderr 0.001;
var v_22; stderr 0.001;
var v_23; stderr 0.001;

// (proxy) noise
var u_r; stderr 0.001;
var u_x; stderr 0.001;
var u_p; stderr 0.001;

end;

 
estimated_params;

// PARAM NAME, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF, UNIFORM_PDF

phi,NORMAL_PDF,2,0.05;
nu,NORMAL_PDF,2,0.05;
psix,NORMAL_PDF,0.5,0.25;
psir,BETA_PDF,0.80,0.25;
psip,Normal_PDF,1.7,1.95;
rhog,BETA_PDF,0.6,0.25;
rhou,BETA_PDF,0.6,0.25;
stderr eg,INV_GAMMA_PDF,0.01,0.05;
stderr eu,INV_GAMMA_PDF,0.01,0.05;
stderr er, INV_GAMMA_PDF,0.01,0.05;

end;

varobs r x p;

% careful: if you use the entire data set the variables are non-stationary and the routine finds that the
% Hessian at the mode is non-positive definite. so use the options first_obs and nobs.
  
//estimation(datafile=rawdata_US_1948Q1_2004q4,first_obs=117,nobs=100,mode_compute=5,mode_check,mh_replic=10000,mh_nblocks=2,mh_jscale=0.65,mh_init_scale=0.5,bayesian_irf,moments_varendo) r, x, p ;
estimation(datafile=rawdata_US_1948Q1_2004q4_L,first_obs=117,nobs=100,mode_compute=5,mode_check,mh_replic=10000,mh_nblocks=2,mh_jscale=0.65,mh_init_scale=0.5,bayesian_irf,moments_varendo) r, x, p ;

% if you have run the program before the previous mode is saved and you can skip the maximization of the posterior using the
% option mode_file=NKMODEL43_mode
% estimation(datafile=rawdata_US_1948Q1_2004q4,first_obs=117,nobs=100,mode_file=NKMODEL_43_mode,mh_replic=10000,mh_nblocks=2,mh_jscale=0.7,mh_init_scale=0.5,bayesian_irf) r x p;
  

% exercise 1: estimate the model with the alternative Taylor rule
% exercise 2: try different mode computing algorithm
% exercise 3: try different values of mh_jscale parameters to see how they affect the convergence rate (try 0.7 and 0.35)
% exercise 4: make the number of replications larger (mh_replication=30000)
% exercise 5: change the initial conditions in the estimation and see  if estimation results 
% exercise 6: changes the variance of the prior (make them larger) and see how results change


