// NEW KEYNESIAN MODEL, Version 2.
// The difference with the original program is that a quadratic trend is taken away
// from all the data (rather than just from output).

// Possibility of running the model with two different Taylor rules
// IS:  x(t) = E_t{x(t+1)} - 1/phi*( r(t) - E_t{p(t+1)} ) + g(t)
// PC:  p(t) = betta*E_t{p(t+1)} + kappa*x(t) + u(t)
// Taylor-Rule 1: r(t) = psir*r(t-1) + (1-psir)*(phix*x(t-1) + psi*p(t-1)) + er(t)
// Taylor-Rule 2: r(t) = psir*r(t-1) + (1-psir)*(phix*x(t) + psi*p(t)) + er(t)
//
//  E(t)=(g(t), u(t), er(t))
//  E(t) = rho*E(t-1) + A*ee(t);  ee ~ nid(0,sigg^2), 
//
// p = inflation, x = output gap, r = interest rate
//
// kappa = lambda * (phi+nu)
// lambda = (1-theta)*(1-betta*theta)/theta

close all; clc;

% preliminary data management
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load gpr.dat;              % data from Ireland (2002)
[nobs,nvar] = size(gpr);   % # Observations in time series
y = gpr(:,1);
pp = gpr(:,2);
rr = gpr(:,3);

//dates = cal(1948,1,4);
// detrend 
cr = ones(nobs,1);
tr = (1:nobs)';
for i=1:nobs
tr2(i) = tr(i)^2;
end;
yt = FOLS(y,[cr tr tr2']);
x = y-yt.beta(2)*tr-yt.beta(3)*tr2';       % output gap
pt = FOLS(pp,[cr tr tr2']);
p = pp-pt.beta(1)-pt.beta(2)*tr-pt.beta(3)*tr2';       % eliminate quadratic trend
rt = FOLS(rr,[cr tr tr2']);
r = rr-rt.beta(1)-rt.beta(2)*tr-rt.beta(3)*tr2';       % eliminate quadratic trend

 
save rawdata_US_1948Q1_2004q4_L  r x p;

 
// statistics on the time series
stats_r  = [mean(r) std(r) min(r) max(r)];
stats_x  = [mean(x) std(x) min(x) max(x)];
stats_p  = [mean(p) std(p) min(p) max(p)];
inv.cnames = strvcat('mean','std','min','max');
inv.rnames = strvcat('series','r','x','p');
disp('*** Summary Statistics of time series ***')
disp('---------------------------------------- ')
// mprint([stats_r;stats_x;stats_p],inv)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


var r x p g u eer;    
varexo eg eu er;  
parameters betta theta phi nu psir psix psip rhog rhou sigg sigu sigr;

% these are calibrated parameters 
betta= 0.99;
theta= 0.75;
  
model(linear); 
          x = x(1)-(1/phi)*(r-p(1))+g;
          p = betta*p(1)+(((1-theta)*(1-betta*theta)/theta)*(phi+nu))*x+u;
//          r = psir*r(-1)+(1-psir)*(psip*p(-1) + psix*x(-1)) + eer;
                    r = psir*r(-1)+(1-psir)*(psip*p + psix*x) + eer; 
          g = rhog*g(-1)+eg;
          u = rhou*u(-1)+eu;
          eer =0*eer(-1)+er;
end; 

shocks;
var eg; stderr 0.1;
var eu; stderr 0.1;
var er; stderr 0.1;
end;

 
estimated_params;

// PARAM NAME, INITVAL, LB, UB, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF

phi,2.0376,0.1,5,NORMAL_PDF,2,0.05;
nu,2.0091,0.1,3,NORMAL_PDF,2,0.05;
psix,-0.0001,-0.1,1.9,NORMAL_PDF,0.5,0.25;
psir,0.8283,0.4,0.99,BETA_PDF,0.80,0.25;
psip,1.4912,1.1,10,Normal_PDF,1.7,1.95;
rhog,0.9841,0.1,0.99999,BETA_PDF,0.6,0.25;
rhou,0.9999,0.1,0.99999,BETA_PDF,0.6,0.25;
stderr eg,0.0015,0.0001,4,INV_GAMMA_PDF,0.01,0.05;
stderr eu,0.0040,0.0001,4,INV_GAMMA_PDF,0.01,0.05;
stderr er,0.0021,0.0001,4,INV_GAMMA_PDF,0.01,0.05;
end;

varobs r x p;

% to make things work use mode_compute=7 (a simplex algorithm). Try also with mode_compute=1 (but need to changes mh_jscale).
% mode_compute=5 (ratto algo) also finds the mode -  it is very slow.
% other algorithms have the problem that the hessian at the mode is not positive definite (can not compute the choleski of it)
% try mh_jscale=0.3 and mh_init_scale=0.1 (to check how they affect the acceptance rate)
% try mh=30000 (to check convergence rate)

estimation(datafile=rawdata2_US_1948Q1_2004q4,mode_compute=7,mode_check,mh_replic=20000,mh_nblocks=2,mh_jscale=0.45,mh_init_scale=0.5,bayesian_irf,moments_varendo) r, x, p ;

%estimation(datafile=rawdata_US_1948Q1_2004q4,mode_file=NKMODEL_41_mode,mh_replic=10000,mh_nblocks=2,mh_jscale=0.7,mh_init_scale=0.5,bayesian_irf);
  
  



